#!/bin/sh

####################################################
#   /media/update/BabyCam.zip 
#   /media/update/BabyCam.md5  
#   /media/update/version.txt
####################################################
SDCARD_PATH=/media
APP_NAME=BabyCam.zip
UPDATE_PATH=/media/update/BabyCam
APP_PATH=/media/update
PKG_NAME=update.zip					
ROOTFS_NAME=rootfs
ROOTFS_FLASHCP=rootfs.squashfs
KERNEL_NAME=kernel					
MD5_CHKSUM_APP=$APP_PATH/BabyCam.md5
FW_PKG_VERSION=$APP_PATH/version.txt
DEV_SYSTEM_PATH=/system
DEV_FW_VERSION=$DEV_SYSTEM_PATH/init/version.txt

remove_others()
{
   /bin/rm -f /system/sound/cn/*
   /bin/rm -f /system/sound/en/*

   /bin/rm -f /system/bin/checkMcuUpgrade
   /bin/rm -f /system/bin/stm32flash

   /bin/rm -f /system/etc/sensor/gc2053*

   /bin/rm -f /system/lib/modules/sensor_gc2053*
   
   /bin/rm -f /system/init/mcu_update.sh

   /bin/echo -e ">>>>>>>>> Remove others done <<<<<<<<<"
}
## remove unused files for space.
func_remove_unittest_files()
{
    ######## check wifi module type: ########
    ## this step should be done after wifi power on.
    WIFI_RTL8811CU=c811
    WIFI_RTL8188EUS=8179
    ##############################################################################
    ## verify wifi module ID by command as follow:
    ##   lsusb | head -1 | awk '{print $6}' |awk -F ":" '{print $2}'
    ##      ==> c811: rtl8811cu.
    ##      ==> 8179: rtl8188eus.
    ##############################################################################
    lsusb
    WIFI_ID=`lsusb | head -1 | awk '{print $6}' | awk -F ":" '{print $2}'`
    /bin/echo -e ">>>>>>>>> WIFI_ID: $WIFI_ID"
    if [ "${WIFI_ID}" = "${WIFI_RTL8188EUS}" ]; then
        echo "-------- matched 8188eus, remove rtl8821cu.ko --------"
        if [ -f /lib/firmware/rtl8821cu.ko ]; then
            /bin/rm -f /system/lib/firmware/rtl8821cu.ko
            echo "-------- <upgrade> matched 8188eus, remove /system/lib/firmware/rtl8821cu.ko --------" >> /system/init/syslog.txt
        fi
        if [ -f /media/update/BabyCam/system/lib/firmware/rtl8821cu.ko ]; then
            /bin/rm -f /media/update/BabyCam/system/lib/firmware/rtl8821cu.ko
            echo "-------- <upgrade> matched 8188eus, remove /media/update/BabyCam/system/lib/firmware/rtl8821cu.ko --------" >> /system/init/syslog.txt
        fi
    else
        echo "-------- matched 8811cu, remove 8188eu.ko --------"
        if [ -f /lib/firmware/8188eu.ko ]; then
            /bin/rm -f /system/lib/firmware/8188eu.ko
            echo "-------- <upgrade> matched 8811cu, remove /system/lib/firmware/8188eu.ko --------" >> /system/init/syslog.txt
        fi
        if [ -f /media/update/BabyCam/system/lib/firmware/8188eu.ko ]; then
            /bin/rm -f /media/update/BabyCam/system/lib/firmware/8188eu.ko
            echo "-------- <upgrade> matched 8811cu, remove /media/update/BabyCam/system/lib/firmware/8188eu.ko --------" >> /system/init/syslog.txt
        fi
    fi
    ########## end. ############

    if [ -f /system/init/wifi_reconnect.sh ]
    then
        /bin/rm -f /system/init/wifi_reconecct.sh
    fi

    remove_others

    /bin/rm -f /system/bin/babymonitor
    /bin/sleep 1
    /bin/sync
    /usr/bin/pkill -9 babymonitor
    echo -e "........remove unittest files done........"
}

## return 0 on success, and 1 on error.
func_check_package_version()
{
    cd $SDCARD_PATH
    tar -xvf $PKG_NAME
    /bin/sync
    cd $APP_PATH
    echo -e "$FW_PKG_VERSION"
    if [ -f $FW_PKG_VERSION ]; ##check /media/update/version.txt existence.
    then
        if [ -f $DEV_FW_VERSION ];
        then
            echo "`cat $DEV_FW_VERSION`"
            CUR_FW_VER=`awk '{print $1}' $DEV_FW_VERSION`
        else
            echo "!!!!!! $DEV_FW_VERSION not found, use default version: 20200101 !!!!!!"
            CUR_FW_VER=20200101
        fi

        echo "`cat $FW_PKG_VERSION`"
        NEW_FW_VER=`awk '{print $1}' $FW_PKG_VERSION`
        echo -e
        echo "====> cur firmware version: $CUR_FW_VER <===="
        echo "====> new firmware version: $NEW_FW_VER <===="
        echo -e

#        if [ $NEW_FW_VER -gt $CUR_FW_VER ]; ##只能往高版本升级
        if [ $NEW_FW_VER -ne $CUR_FW_VER ]; ##可以版本回溯
        then
            echo "<><><><><><> there exist new firmware version, need to upgrade <><><><><><>"
            return 0
        else
            echo "============ there is no need to upgrade ============"
            /bin/rm -rf $APP_PATH
            /bin/sync 
            return 1
        fi
    else ## version.txt not existed.
        echo  "!!!!!!!!! Warning, $FW_PKG_VERSION is not found !!!!!!!!!"
        /bin/rm -rf $APP_PATH
        /bin/sync 
        return 1;
    fi
}

## return 0 on success, and 1 on error.
func_check_md5_and_update()
{
    cd $APP_PATH
    echo -e "$MD5_CHKSUM_APP"
    if [ -f $MD5_CHKSUM_APP ]; ##check BabyCam.md5 existence.
    then
        echo "`cat $MD5_CHKSUM_APP`"
        UPDATE_CHECKSUM=`awk '{print $1}' $MD5_CHKSUM_APP`
        UPDATE_PACKAGENAME=`awk '{print $2}' $MD5_CHKSUM_APP`
        echo "+++ update check_sum     : $UPDATE_CHECKSUM"
        echo "+++ update package name  : $UPDATE_PACKAGENAME"

        if [ -f $APP_PATH/$UPDATE_PACKAGENAME ] && [ $APP_NAME == $UPDATE_PACKAGENAME ]; ##check BabyCam.zip existence.
        then
            /usr/bin/md5sum $APP_NAME > /tmp/tmp.md5  ##generate new md5 checksum.
            echo -e "`cat /tmp/tmp.md5`"
            NEW_CHECK_SUM=`awk '{print $1}' /tmp/tmp.md5`
            echo -e "+++ new check_sum  : $NEW_CHECK_SUM"

            if [ $UPDATE_CHECKSUM == $NEW_CHECK_SUM ];
            then
		touch /tmp/led_updating.flag
                echo -e ">>>>>> [$UPDATE_PACKAGENAME] md5 matched success, start to update >>>>>>"
                /bin/tar -xvf $APP_PATH/$APP_NAME -C $APP_PATH
                sync

                echo -e ">>>>>> check kernel & rootfs existence."
                ##check kernel existence, and copy to /media/.
                if [ -f $UPDATE_PATH/kernel ]; then
                    echo -e ">>>>>> copy kernel to /media/ ......"
                    cp -f $UPDATE_PATH/kernel $SDCARD_PATH
                    if [ $? -ne 0 ]; then
                        echo "!!!!!!!!!!!! copy kernel failed, exit !!!!!!!!!!!!"
                        return 1;
                    fi
                fi

                ##check rootfs existence, and copy to /media/.
                if [ -f $UPDATE_PATH/rootfs ]; then
                    echo -e ">>>>>> copy rootfs to /media/ ......"
                    cp -f $UPDATE_PATH/rootfs $SDCARD_PATH
                    if [ $? -ne 0 ]; then
                        echo "!!!!!!!!!!!! copy rootfs failed, exit !!!!!!!!!!!!"
                        return 1;
                    fi
                fi

                if [ -f $UPDATE_PATH/rootfs.squashfs ]; then
                    echo -e ">>>>>> copy rootfs.squashfs to /media/ ......"
                    cp -f $UPDATE_PATH/rootfs.squashfs $SDCARD_PATH
                    if [ $? -ne 0 ]; then
                        echo "!!!!!!!!!!!! copy rootfs.squashfs failed, exit !!!!!!!!!!!!"
                        return 1;
                    fi
                fi

                func_remove_unittest_files
                echo -e "   >>>>>>>>>> cover files into $DEV_SYSTEM_PATH <<<<<<<<<    "
                cp -Rfa $UPDATE_PATH/system/* $DEV_SYSTEM_PATH
                if [ $? -ne 0 ]; then
                    echo "!!!!!!!!!!!! copy /system/* failed, exit !!!!!!!!!!!!"
                    return 1;
                fi

                /bin/chmod -R a+x $DEV_SYSTEM_PATH/bin
                cp -f $FW_PKG_VERSION $DEV_SYSTEM_PATH/init/
                /bin/sync
                return 0;    ##succuss.
            else
                echo -e "!!!!!!!! md5 check failed, exit !!!!!!!!"
                return 1;   ##check md5 failed.
            fi
        else ##BabyCam.zip not existed.
            echo -e "!!! Error, $APP_PATH/$APP_NAME is not found !!!!!!"
            return 1;
        fi
    else ## BabyCam.md5 not existed.
        echo -e "!!!!!!!!! Warning, [$MD5_CHKSUM_APP] is not found !!!!!!!!!"
        return 1;
    fi
}

do_system_restart()
{
    /bin/sleep 1
    /bin/sync
#    /bin/rm -rf $SDCARD_PATH/$PKG_NAME
    /bin/rm -rf $APP_PATH
    /bin/rm -rf /system/init/silence.flag
    /bin/sync 
    echo -e
    echo -e ">>>>>>>>> System Restart Now...... "
    echo -e
    /sbin/reboot
}


## main ##
if [ -f /system/bin/impdbg ]; then
    /bin/rm -f /system/bin/impdbg
fi

if [ -f /system/bin/rtwpriv ]; then
    /bin/rm -f /system/bin/rtwpriv
fi

if [ -f /system/bin/iperf ]; then
    /bin/rm -f /system/bin/iperf
fi

if [ -f /system/bin/stm32flash ]; then
    /bin/rm -f /system/bin/stm32flash
fi

if [ -f /system/bin/checkMcuUpgrade ]; then
    /bin/rm -f /system/bin/checkMcuUpgrade
fi

echo -e ">>> $SDCARD_PATH/$PKG_NAME"
if [ -f $SDCARD_PATH/$ROOTFS_NAME ]; 
then
    touch /tmp/led_updating.flag
    if [ -f /system/bin/self_flashcp ];
    then
        /system/bin/self_flashcp
    else
        echo -e "self_flascp is not exist"
    fi
    #flashcp -v /media/rootfs /dev/mtd2
    echo -e "++++++++++++++ delete rootfs +++++++++++++++";
    rm -f $SDCARD_PATH/$ROOTFS_NAME  
    /sbin/reboot
else
    echo -e "++++++++++ rootfs not found ++++++++++"
fi

# if [ -f $SDCARD_PATH/$ROOTFS_FLASHCP ]; 
# then
#     flashcp -v /media/rootfs.squashfs /dev/mtd2
#     echo -e "++++++++++++++ delete rootfs +++++++++++++++";
#     rm -f $SDCARD_PATH/$ROOTFS_FLASHCP  
#     /sbin/reboot
# else
#     echo -e "++++++++++ rootfs not found ++++++++++"
# fi

if [ -f $SDCARD_PATH/$KERNEL_NAME ]; 
then
    echo -e "++++++++++++++ delete kernel +++++++++++++++";
    rm -f $SDCARD_PATH/$KERNEL_NAME  
else
    echo -e "++++++++++ kernel not found ++++++++++"
fi

if [ -f $SDCARD_PATH/$PKG_NAME ]; 
then
    echo -e "++++++++++++++ babycam Upgrade Start +++++++++++++++";
else
    echo -e "++++++++++ [update.zip] not found, there is no need to upgrade ++++++++++"
    exit
fi

#echo "==> step 1: check md5 for package......"
echo "==> step 1: check version for package......"
func_check_package_version
if [ $? == 0 ];
then
    echo -e ">>>>>>>>> need update <<<<<<<<<"
else
    echo -e "!!!!!! no need to update, exit !!!!!!"
    exit
fi

func_check_md5_and_update
if [ $? == 0 ];
then
    echo -e ">>>>>>>>> update success <<<<<<<<<"
    #rm -rf /system/init/BakPose.motor
    do_system_restart
else
    echo -e "!!! Error, ret_val = $?, check md5 for update package failed, exit !!!!!!"
    exit
fi
